

UNIV_DECON <- function(ws,mis,K.x,K.u,K.t,simsize,burnin,plot_results=TRUE)
{
	######################################
	### Add Libraries and Source Files ###
	######################################

# These libraries must be available to R
#library(mvtnorm,lib.loc="~/RLibs") 
#library(msm,lib.loc="~/RLibs")
library(mvtnorm) 
library(msm)



	#################################
	### Priors and Initial Values ###
	#################################

### Initialization and prior of xs and us
n <- length(mis)
N <- sum(mis)
inds <- rep(1:n,times=mis)
wbars <- tapply(ws,inds,"mean")  
xs <- current.xs <- start.xs <- as.vector(wbars)
us <- ws - rep(xs,times=mis)
range.start.xs <- diff(range(start.xs))
s2is <- as.vector(tapply(ws,inds,var))
x.grid <- seq(min(xs,0)-.1*range.start.xs,max(xs)+.1*range.start.xs,length=500)

alpha.x = 1

# Normal-Inverse-Scaled-Chisquare
nu0.x = 1/5
gama0.x = 5	                  			# Note the spelling, also note that this must be greater than 2
mu0.x = mean(xs)
sigmasq0.x = 1*(gama0.x-2)/gama0.x		#var(xs)/((1+1/nu0.x)*(gama0.x/(gama0.x-2)))
student.x.grid = (x.grid-mu0.x)/(sqrt(sigmasq0.x)*sqrt(1+1/nu0.x))

Pi.x <- rep(1/K.x,K.x)
clusters.x <- kmeans(xs,K.x)
mu.x <- clusters.x$center
C.x = clusters.x$cluster
sigmasq.x <- rep(2*var(xs)/K.x,K.x)

d.ordinates.x = matrix(0,nrow=n,ncol=K.x)


### Prior and initialization of s2t and thetas

alpha.t <- .1
beta.t <- .1

s2t <- current.s2t <- start.s2t <- 0.01

P.t <- P.mat(K.t+1) 	# penalty matrix
knots.t <- seq(min(start.xs,0)-.1*range.start.xs,max(start.xs)+.1*range.start.xs,length=K.t)

optim_results <- optim(rep(1,K.t+1), fr, NULL, xs, mis, knots.t, P.t, s2t, us, method = "BFGS")
thetas <- current.thetas <- start.thetas <- optim_results$par
vfix <- exp(thetas[2])+exp(thetas[3])
prop.sig.thetas <- prop.sig.thetas.fn(thetas,xs,mis,us,s2t,K.t,P.t,knots.t,n)

var.grid <- seq(min(start.xs)-.1*range.start.xs,max(start.xs)+.1*range.start.xs,length=100)
vars <- current.vars <- B.basis(xs,knots.t)%*%exp(current.thetas)
es <- us/rep(B.basis(xs,knots.t)%*%exp(start.thetas),times=mis)

### Prior and initialization for mixture
simsize.mh.u <- 20
z.u <- rep(1,N)
alpha.u <- 0.1
params.u <- matrix(c(0.5,0,1,1),nrow=K.u,ncol=4,byrow=T)		# unique values
Pi.u <- rep(1/K.u,K.u)
d.ordinates.u <- matrix(0,nrow=N,ncol=K.u)



	#########################
	### Tuning Parameters ###
	#########################


sig.tune.thetas.1 <- 0
sig.tune.thetas.2 <- 0.1





	###############################
	### Storage for MCMC Output ###
	###############################


e.grid <- seq(-3,3,length=500)
density.x.est <- numeric(length(x.grid))
var.est <- numeric(length(var.grid))
density.e.est <- numeric(length(e.grid))
accepted <- numeric(simsize)
Mat.thetas <- matrix(0,nrow=simsize-burnin,ncol=length(thetas))
postmean.xs <- numeric(n)



	##################
	### Start MCMC ###
	##################



for (iii in 1:simsize)
	{
	if(iii%%100==0)
		print(iii)



	### Updating C.x
	for(kk in 1:K.x)
		d.ordinates.x[,kk] = dnorm(xs,mu.x[kk],sqrt(sigmasq.x[kk]))
	for(ii in 1:n)
		C.x[ii] = sample(K.x,1,prob=Pi.x*d.ordinates.x[ii,])

	
	### Updating cluster probabilities
	n.kk.x = tabulate(C.x,nbins=K.x)
	Pi.x = rdirichlet(1,alpha.x/K.x+n.kk.x)	


	### Updating mu.x, sigmsq.x
	
	for(kk in 1:K.x)
		{
		temp = which(C.x==kk)
		xspool = xs[temp]

		nutemp.x = nu0.x + length(xspool)
		mutemp.x = (nu0.x*mu0.x + sum(xspool)) / (nutemp.x)
		gamatemp.x = gama0.x + length(xspool)				# Normal-Inverse-Scaled-Chisquare
		sigmasqtemp.x = (gama0.x*sigmasq0.x + sum(xspool^2) + nu0.x*mu0.x^2 - nutemp.x*mutemp.x^2) / gamatemp.x	
		sigmasq.x[kk] = 1/rgamma(1,shape=gamatemp.x/2,rate=(gamatemp.x*sigmasqtemp.x)/2)
		mu.x[kk] = rnorm(1,mutemp.x,sqrt(sigmasq.x[kk]/nutemp.x))
		}









	### Updating xs (and us)

	proposed.xs = rtnorm(n,mean=current.xs,sd=diff(range(start.xs))/6,lower=min(knots.t),upper=max(knots.t))
	proposed.vars = B.basis(proposed.xs,knots.t)%*%exp(thetas)
		
	proposed.prior = current.prior = numeric(n)
	for(kk in 1:K.x)
		{
		proposed.prior = proposed.prior + Pi.x[kk]*dnorm(proposed.xs,mu.x[kk],sqrt(sigmasq.x[kk]))
		current.prior = current.prior + Pi.x[kk]*dnorm(current.xs,mu.x[kk],sqrt(sigmasq.x[kk]))
		}

	mh.ratio = proposed.prior/current.prior
	mh.ratio = (   mh.ratio * dtnorm(current.xs,mean=proposed.xs,sd=diff(range(start.xs))/6,lower=min(knots.t),upper=max(knots.t))
					/ dtnorm(proposed.xs,mean=current.xs,sd=diff(range(start.xs))/6,lower=min(knots.t),upper=max(knots.t))   )

	temp.current.likelihood = d.restricted.mix.norm(ws,mean=rep(current.xs,times=mis),sd=rep(sqrt(current.vars),times=mis),params.u[z.u,])
	temp.proposed.likelihood = d.restricted.mix.norm(ws,mean=rep(proposed.xs,times=mis),sd=rep(sqrt(proposed.vars),times=mis),params.u[z.u,])
		
	current.likelihood = tapply(temp.current.likelihood,inds,"prod")
	proposed.likelihood = tapply(temp.proposed.likelihood,inds,"prod")

	mh.ratio = mh.ratio * proposed.likelihood/current.likelihood	
	mh.ratio[is.nan(mh.ratio)] = 0

	u = runif(n)
	inds.to.replace =(1:n)[u<mh.ratio]
	xs[inds.to.replace] = current.xs[inds.to.replace] = proposed.xs[inds.to.replace]
	vars[inds.to.replace] = current.vars[inds.to.replace] = proposed.vars[inds.to.replace]

	us = ws - rep(xs,times=mis)
		

	### Updating theta

	proposed.thetas = rmvnorm(1,current.thetas,(diag(rep(sig.tune.thetas.1,(K.t+1)))+sig.tune.thetas.2*prop.sig.thetas))
 	if(F){while(proposed.thetas[3]>log(vfix))
		proposed.thetas = rmvnorm(1,current.thetas,(diag(rep(sig.tune.thetas.1,(K.t+1)))+sig.tune.thetas.2*prop.sig.thetas))
	proposed.thetas[2] = log(vfix-exp(proposed.thetas[3]))}
	proposed.thetas = t(proposed.thetas)
	proposed.vars = B.basis(xs,knots.t)%*%exp(proposed.thetas)
		
	current.log.prior = - t(current.thetas)%*%P.t%*%current.thetas/(2*s2t)
	proposed.log.prior = - t(proposed.thetas)%*%P.t%*%proposed.thetas/(2*s2t)

	temp.current.likelihood = d.restricted.mix.norm(us,mean=rep(0,times=N),sd=rep(sqrt(current.vars),times=mis),params.u[z.u,])
	temp.proposed.likelihood = d.restricted.mix.norm(us,mean=rep(0,times=N),sd=rep(sqrt(proposed.vars),times=mis),params.u[z.u,])

	current.log.likelihood = sum(log(temp.current.likelihood))
	proposed.log.likelihood = sum(log(temp.proposed.likelihood))

	log.mh.ratio = proposed.log.prior + proposed.log.likelihood - current.log.likelihood - current.log.prior

	log.u = log(runif(1))
	if(log.u<log.mh.ratio)
		{
		thetas = current.thetas = proposed.thetas
		vars = current.vars = proposed.vars
		}
	if(iii>burnin) 
		Mat.thetas[iii-burnin,] = thetas



	### Updating z.u
	for(ii in 1:N)
		for(kk in 1:K.u)
			d.ordinates.u[ii,kk] = d.restricted.mix.norm(us[ii],mean=0,sd=sqrt(vars[inds[ii]]),params.u[kk,])
	for(ii in 1:N)
		z.u[ii] = sample(K.u,1,prob=Pi.u*d.ordinates.u[ii,])
	
	### Updating cluster probabilities
	n.kk.u = tabulate(z.u,nbins=K.u)
	Pi.u = rdirichlet(1,alpha.u/K.u+n.kk.u)	
    
	### Updating params.u
	
	if(iii>200){simsize.mh.u = 1}
	for(rr in 1:simsize.mh.u)
	{
	for(kk in which(n.kk.u>0))
		{
		temp = which(z.u==kk)

		proposed.params.u <- r.proposal.params.restricted.mix.norm(1,1,3,3,3,3,3)

		current.log.likelihood = sum(log(d.restricted.mix.norm(us[temp],mean=0,sd=sqrt(vars[inds[temp]]),params.u[kk,])))
		proposed.log.likelihood = sum(log(d.restricted.mix.norm(us[temp],mean=0,sd=sqrt(vars[inds[temp]]),proposed.params.u)))

		log.mh.ratio = proposed.log.likelihood - current.log.likelihood
		if(log(runif(1))<log.mh.ratio)
			params.u[kk,] <- proposed.params.u
		}
	}	
	for(kk in which(n.kk.u==0))
		{
		params.u[kk,] <- r.proposal.params.restricted.mix.norm(1,1,3,3,3,3,3)		
		}


	### Updating s2t
	s2t = 0.1	





	if(iii>burnin)
		{
		postmean.xs = postmean.xs + xs
		for(kk in 1:K.x)
			density.x.est = density.x.est + Pi.x[kk]*dnorm(x.grid,mu.x[kk],sqrt(sigmasq.x[kk]))
		
		k.u = max(z.u)                # Number of clusters
		prob.e <- tabulate(z.u)/N
		var.e = var.e.fn(prob.e[1:k.u],params.u[1:k.u,])
		var.est = var.est + B.basis(var.grid,knots.t)%*%exp(thetas) * var.e
		density.e.est <- density.e.est + d.scaled.restricted.mix.norm(e.grid,0,1,prob.e[1:k.u],params.u[1:k.u,])
		}
	
	}



density.x.est <- density.x.est/(simsize-burnin)
density.e.est <- density.e.est/(simsize-burnin)
postmean.xs <- postmean.xs/(simsize-burnin)

var.true <- (1+var.grid/4)^2
var.est <- var.est/(simsize-burnin)

fr1 <- function(thetas)
	{y = sum((var.est-B.basis(var.grid,knots.t)%*%exp(thetas))^2)}
optim_results <- optim(colMeans(Mat.thetas), fr1, method = "BFGS")
thetas.final <- optim_results$par

xs.final <- postmean.xs
es.final <- (ws-rep(xs.final,times=mis))/rep(sqrt(B.basis(xs.final,knots.t)%*%exp(thetas.final)),times=mis)

if(plot_results==TRUE)
	{
	par(mfrow=c(3,1))
	plot(x.grid,density.x.est,type="l",lty=1,col="green3",lwd=3)


	plot(e.grid,density.e.est,type="l",lty=1,col="green3",lwd=3)
	points(e.grid,dnorm(e.grid),type="l",lty=1)


	plot(xs,s2is,pch="*")
	points(var.grid,B.basis(var.grid,knots.t)%*%exp(thetas.final),type="l",lty=1,col="green3",lwd=2)
	par(mfrow=c(1,1))
	}



return(list(knots=knots.t, thetas=thetas.final, xs=xs.final, es=es.final))
}

